
#ifndef VTKIMAGINGMORPHOLOGICAL_EXPORT_H
#define VTKIMAGINGMORPHOLOGICAL_EXPORT_H

#ifdef VTKIMAGINGMORPHOLOGICAL_STATIC_DEFINE
#  define VTKIMAGINGMORPHOLOGICAL_EXPORT
#  define VTKIMAGINGMORPHOLOGICAL_NO_EXPORT
#else
#  ifndef VTKIMAGINGMORPHOLOGICAL_EXPORT
#    ifdef ImagingMorphological_EXPORTS
        /* We are building this library */
#      define VTKIMAGINGMORPHOLOGICAL_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKIMAGINGMORPHOLOGICAL_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKIMAGINGMORPHOLOGICAL_NO_EXPORT
#    define VTKIMAGINGMORPHOLOGICAL_NO_EXPORT 
#  endif
#endif

#ifndef VTKIMAGINGMORPHOLOGICAL_DEPRECATED
#  define VTKIMAGINGMORPHOLOGICAL_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKIMAGINGMORPHOLOGICAL_DEPRECATED_EXPORT
#  define VTKIMAGINGMORPHOLOGICAL_DEPRECATED_EXPORT VTKIMAGINGMORPHOLOGICAL_EXPORT VTKIMAGINGMORPHOLOGICAL_DEPRECATED
#endif

#ifndef VTKIMAGINGMORPHOLOGICAL_DEPRECATED_NO_EXPORT
#  define VTKIMAGINGMORPHOLOGICAL_DEPRECATED_NO_EXPORT VTKIMAGINGMORPHOLOGICAL_NO_EXPORT VTKIMAGINGMORPHOLOGICAL_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKIMAGINGMORPHOLOGICAL_NO_DEPRECATED
#    define VTKIMAGINGMORPHOLOGICAL_NO_DEPRECATED
#  endif
#endif

#endif /* VTKIMAGINGMORPHOLOGICAL_EXPORT_H */
